@@ -18,6 +18,10 @@ except ImportError:
 
 import simplejson
 import subprocess
+import sys
+import time
+
+SAUCE_CONNECT_TIMEOUT = 60
 
 FIREFOX = 'firefox'
 IE = 'ie'
@@ -38,8 +42,8 @@ class Selenium(SeleniumBase):
     For more information, see http://saucelabs.com/docs/sauce-connect
     """
     
-    def __init__(self, host, port, browser, browserURL, sauceUsername, sauceApiKey, 
-                 sauceDomain=None, sauceConnect='sauce_connect', os=LINUX, browserVersion='',
+    def __init__(self, host, port, browser, sauceUsername, sauceApiKey, sauceDomain,
+                 browserURL='http://saucelabs.com', sauceConnect='sauce_connect', os=LINUX, browserVersion='',
                  build=None, customData={}):
         """
         Additional parameters for Sauce OnDemand:
@@ -48,6 +52,9 @@ class Selenium(SeleniumBase):
         - os: operating system name
         - browserVersion: browser version number
         """
+        assert sauceUsername, 'sauceUsername cannot be empty'
+        assert sauceApiKey, 'sauceApiKey cannot be empty'
+
         self.sauceUsername = sauceUsername
         self.sauceApiKey = sauceApiKey
         self.sauceConnect = sauceConnect
@@ -79,22 +86,37 @@ class Selenium(SeleniumBase):
         "Starts the Sauce OnDemand tunnel with sauce-connect."
 
         cmd = [self.sauceConnect, '-u', self.sauceUsername, '-k', self.sauceApiKey,
-               '-s', self.host, '-p', self.port]
-        
-        if self.sauceDomain:
+               '-s', self.serverHost, '-p', self.serverPort, '-t', self.serverPort]
+        if isinstance(self.sauceDomain, (list, tuple)):
+            for domain in self.sauceDomain:
+                cmd.extend(['-d', domain])
+        else:
             cmd.extend(['-d', self.sauceDomain])
         
-        self.sauceTunnel = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        self.sauceTunnel = subprocess.Popen(map(str, cmd), stdout=subprocess.PIPE)
+        start = time.time()
+        while True:
+            line = self.sauceTunnel.stdout.readline()
+            sys.stdout.write(line)
+            if line.strip().endswith('You may start your tests.'):
+                break
+            if time.time() - start > SAUCE_CONNECT_TIMEOUT:
+                self.stop_sauce_tunnel()
+                raise Exception('sauce_connect failed to come online in %s seconds' % SAUCE_CONNECT_TIMEOUT)
     
     def stop_sauce_tunnel(self):
         self.sauceTunnel.terminate()
+        self.sauceTunnel.wait()
 
     def start(self, *args, **kwargs):
         "Initiates a sauce tunnel followed by a selenium instance."
 
         self.start_sauce_tunnel()
 
-        SeleniumBase.start(self, *args, **kwargs)
+        result = self.get_string("getNewBrowserSession", [self.browserStartCommand, self.browserURL, self.extensionJs])
+
+        self.sessionId = result
+        print >> sys.stdout, "SauceOnDemandSessionID=" + result
 
     def stop(self, *args, **kwargs):
         "Completes Sauce OnDemand tunnel connection."
